@@ -1,5 +1,5 @@
 #
-# $Id: Engine.pm,v 1.8 1999/03/06 20:07:00 jsmith Exp $
+# $Id: Engine.pm,v 1.9 1999/03/19 03:38:05 jsmith Exp $
 #
 # Author: James G. Smith
 #
@@ -29,7 +29,7 @@ use Carp;
 use vars qw($VERSION);
 use integer;
 
-$VERSION = '0.6.2';
+$VERSION = '0.6.3';
 
 =pod
 
@@ -235,8 +235,14 @@ If using mod_perl, the following variables may be set using B<PerlSetVar>:
    WeT_Email - email of the administrator
    WeT_ProblemsEmail - email for bug reports and other problems
    WeT_DefaultTheme - initial theme people will see
-   WeT_UseSSLURLs - (yes or no) - whether to add :SSL and :NOSSL to the
-                    end of URLs generated by the engine or not.
+   WeT_UseSSLURLs - whether to add :SSL and :NOSSL to the
+                    end of URLs generated by the engine or not. (This
+                    is considered true if defined.)  This is still
+                    experimental and will most likely break graphical
+                    navigation.
+   WeT_AnonymousCoward - in a bow to slashdot.org, this is what the site
+                         names anonymous contributors.  This is not
+                         retroactive (yet).
 
 Otherwise, the function B<CGI::WeT::site_config> must be defined expecting
 a reference to the engine object.  The following members of the object need
@@ -249,7 +255,7 @@ to be defined:
    $engine->{'PROBLEMS_EMAIL'} -- corresponds to WeT_ProblemsEmail
    $engine->{'DEFAULT_THEME'} -- corresponds to WeT_DefaultTheme
    $engine->{'SSL_URLS'} -- corresponds to WeT_UseSSLURLs
-
+   $engine->{'AC'} -- corresponds tp WeT_AnonymousCoward
 =cut
 
 	# get config from httpd.conf
@@ -265,9 +271,10 @@ to be defined:
 	    $r->dir_config('WeT_ProblemsEmail') || $self->{'EMAIL'};
 	$self->{'DEFAULT_THEME'} = $r->dir_config('WeT_DefaultTheme') ||
             'plain';
-        $self->{'SSL_URLS'} = ($r->dir_config('WeT_UseSSLURLs') eq 'yes');
+        $self->{'SSL_URLS'} = $r->dir_config('WeT_UseSSLURLs');
+        $self->{'AC'} = $r->dir_config('WeT_AnonymousCoward');
         foreach (map((m/(.*)::$/),
-                 grep(!/^[A-Z]*$/ && /^[A-Z]/, keys %CGI::WeT::Modules::))) {
+                 grep(!/^[A-Z_]*$/ && /^[A-Z]/, keys %CGI::WeT::Modules::))) {
             no strict;
             if(defined & { "CGI::WeT::Modules::$_\::initialize" }) {
                 & { "CGI::WeT::Modules::$_\::initialize" } ($self, $r);
@@ -285,6 +292,7 @@ to be defined:
             $self->{'EMAIL'} = $ENV{'SERVER_ADMIN'};
             $self->{'PROBLEMS_EMAIL'} = $self->{'EMAIL'};
             $self->{'DEFAULT_THEME'} = 'plain';
+            $self->{'AC'} = 'Anonymous Coward';
         }
     }
 
@@ -503,15 +511,26 @@ sub header {
 
 =item $engine->body_push(B<array>)
 
+=item $engine->print(B<array>)
+
 This function places B<array> at the end of the body content caching it for
 later use by the rendering code.  Text placed in the body cannot be
 removed by a provided method.
 
+Use of B<body_push> is depricated in favor of B<print> and will be removed in
+the 0.7 release.
+
 =cut
 
 sub body_push {
     my $self = shift;
 
+    $self->print(@_);
+}
+
+sub print {
+    my $self = shift;
+
     push @{ $$self{BODY} }, @_;
 }
 
@@ -544,10 +563,15 @@ sub url {
     $url =~ s!//!/!g;
 
     if($self->{'SSL_URLS'}) {
-	if($url =~ /\.private\./ || $url =~ /\.form\./) {
-	    $url .= ":SSL";
+	$url =~ s/:(NO)SSL//g;
+	if($ENV{'SSL_PROTOCOL'}) {
+	    if($url !~ /\.private\./ && $url !~ /\.form\./) {
+		$url .= ":NOSSL";
+	    }
 	} else {
-	    $url .= ":NOSSL";
+	    if($url =~ /\.private\./ || $url =~ /\.form\./) {
+		$url .= ":SSL";
+	    }
 	}
     }
 
@@ -569,6 +593,7 @@ sub filename {
     my $url = join("", @_);
 
     $url =~ s/;//g;
+    $url =~ s/:(NO)SSL$// if($engine->{'SSL_URLS'});
 
     if($engine->{'MOD_PERL'}) {
 	my $r = Apache->request;
@@ -627,11 +652,29 @@ sub render_content {
                 $self->arguments_push($args);
 		no strict;
                 if(defined $self->{'THEME'}->page_type($object)) {
-		    $self->content_push($self->{'THEME'}->page_type($object)
-					->LAYOUT);
+		    my $pt = $self->{'THEME'}->page_type($object);
+		    $self->content_push($pt->LAYOUT || [ '' ]);
                     push(@output, $self->render_content);
+		    if($pt->SCRIPTS) {
+			my $s;
+			foreach $s (keys % { $pt->SCRIPTS }) {
+			    if($self->argument("\L$s") ne 'no') {
+				$self->{'SCRIPTS'}->{$s}->{$object} =
+				    $pt->SCRIPTS->{$s};
+			    }
+			}
+		    }
                 } elsif(defined & { "CGI::WeT::Modules::$object" }) {
                     push(@output, & { "CGI::WeT::Modules::$object" }($self));
+		    if(defined & { "CGI::WeT::Scripts::$object" }) {
+			my $sk = & { "CGI::WeT::Scripts::$object" }($self);
+			foreach $s (keys %$sk) {
+			    if($self->argument("\L$s") ne 'no') {
+				$self->{'SCRIPTS'}->{$s}->{$object} =
+				    $sk->{$s};
+			    }
+			}
+		    }
                 }
 		$self->arguments_pop;
             } else {
@@ -657,11 +700,10 @@ by B<new CGI::WeT::Theme> or a derived class of B<CGI::WeT::Theme>.
 sub render_page {
     my $self = shift;
     $self->{'THEME'} = shift;   # will be undef if no theme specified...
-    my(@output);
+    my(@output, @head);
     my($theme, $layout, $css);
 
-    push(@output, "<html>", "<head>");
-    push(@output,
+    push(@head,
          map("<meta name=\"$_\" content=\"" . scalar($self->header($_)) 
 	     . "\">",
              grep(defined $$self{HEADERS}->{$_},
@@ -669,12 +711,12 @@ sub render_page {
                   )
              )
          );
-    push(@output, "<meta name=\"Generator\" content=\"CGI::WeT $CGI::WeT::Engine::VERSION\">");
-    push(@output, "<meta name=\"Theme\" content=\"", 
+    push(@head, "<meta name=\"Generator\" content=\"CGI::WeT $CGI::WeT::Engine::VERSION\">");
+    push(@head, "<meta name=\"Theme\" content=\"", 
 	 $self->argument('theme'),
 	 "\">");
 
-    push(@output, "<title>", "$$self{SITENAME} - ", 
+    push(@head, "<title>", "$$self{SITENAME} - ", 
 	 scalar($self->header('Title')),
          "</title>");
 
@@ -684,9 +726,9 @@ sub render_page {
     $layout = $self->{'THEME'}->page_type(($self->header('Type')), 'DEFAULT');
 
     if($layout->has_css) {
-	push(@output, "<style type=\"text/css\">", "<!-- ");
-	push(@output, @ { $layout->CSS });
-	push(@output, " -->","</style>", "</head>");
+	push(@head, "<style type=\"text/css\">", "<!-- ");
+	push(@head, @ { $layout->CSS });
+	push(@head, " -->","</style>");
     }
     if(defined $layout->BODY) { 
 	my($bodyinfo) = $layout->BODY;
@@ -704,43 +746,64 @@ sub render_page {
 
     $self->content_push($layout->LAYOUT);
     push(@output, $self->render_content);
-    push(@output, "</body>","</html>");
-    return @output;
+    push(@output, "</body>");
+    my $sk;
+    foreach $sk (keys % { $self->{'SCRIPTS'} }) {
+	if($self->{'SCRIPTS'}->{$sk}) {
+	    push(@head, "<script language=\"$sk\">", "<!--\n");
+	    my $jk;
+	    foreach $jk (keys % { $self->{'SCRIPTS'}->{$sk} }) {
+		push(@head, "// for $jk\n",
+		     join("\n", @ { $self->{'SCRIPTS'}->{$sk}->{$jk} }));
+	    }
+	    push(@head, "\n// -->", "</script>");
+	}
+    }
+    
+    return ("<html>", "<head>", @head, "</head>", @output, "</html>");
 }
 
 sub handler {
     my $r = shift;
     my $filename = $r->filename;
-    my $engine = new CGI::WeT::Engine;
     my ($key, $val);
 
-    if(-e $r->finfo) {
-        open IN, "<$filename";
-        while(<IN>) {
-            last if /^\s*$/;
-            next if /^\s*#/;
-            chomp;
-            if(/^[A-Za-z_]+:/) {
-                ($key, $val) = split(/:\s*/,$_,2);
-            } else {
-                $val = $_;
-            }
-            $engine->headers_push($key, $val);
-        }
-
-        #
-        # slurp up the rest of the file
-        #
-        $engine->body_push((<IN>));
-        close IN;
+    {
+	no strict;
+	return Apache::Constants::NOT_FOUND
+	    unless open IN, "<$filename";
     }
 
-    $r->send_cgi_header(<<EOF);
-Content-type: text/html
+    my $engine = new CGI::WeT::Engine;
 
-EOF
+    while(<IN>) {
+	last if /^\s*$/;
+	next if /^\s*#/;
+	chomp;
+	if(/^[A-Za-z_]+:/) {
+	    ($key, $val) = split(/:\s*/,$_,2);
+	} else {
+	    $val = $_;
+	}
+	$engine->headers_push($key, $val);
+    }
+    
+    #
+    # slurp up the rest of the file
+    #
+    $engine->body_push((<IN>));
+    close IN;
+    
+    $r->content_type('text/html');
+
+    $r->send_http_header;
 
     $r->print( $engine->render_page );
+
+    {
+	no strict;
+	return Apache::Constants::OK;
+    }
 }
 
 1;
